#include "testScene.h"
#include "testApp.h"

//---------------------------------------
testScene::testScene(){

}

//---------------------------------------
void testScene::setupScene(){
	sceneName = "my test scene!";
	addAppListener();
	addMouseListener();
	addKeyboardListener();
	setAutoRender(true);
	
	//load the plane from xml
	xml.loadFile("planeSettings.xml");
	
	planePts[0].x = xml.getValue("A:x", 10);
	planePts[0].y = xml.getValue("A:y", 10);
	planePts[1].x = xml.getValue("B:x", 100);
	planePts[1].y = xml.getValue("B:y", 10);
	planePts[2].x = xml.getValue("C:x", 100);
	planePts[2].y = xml.getValue("C:y", 100);	
	planePts[3].x = xml.getValue("D:x", 100);
	planePts[3].y = xml.getValue("D:y", 10);	
	int planeMM	  = xml.getValue("planeMM", 300);	
	
	plane.setup(camW, camH, planeMM, true);
	
	fakeMarker.setup(camW, camH, 60, true);
	
	plane.setMarkerPoints(planePts[0], planePts[1], planePts[2], planePts[3]);
	plane.setMarkerSize(planeMM);
	
	mx = 0;
	my = 0;
	
	//this is for setting the points with your mouse
	whichPt		 = 0;
	recordingPts = false;
	
}

//---------------------------------------
void testScene::render(){
	if( hide || pause ) return;
	
	//draw a circle for each card found
	if( plane.isMarkerSet()){
		plane.loadARMatrix();
		
			ofSetColor(0xFF0000);
		
		
		
			
			if( currentCards.size() > 0){
				for(int i = 0; i < currentCards.size(); i++){
					
					/*
					 
					JUMP OFF THE PAGE
					 
					fakeMarker.setMarkerPoints(currentCards[i].sqPts[0], currentCards[i].sqPts[1], currentCards[i].sqPts[2], currentCards[i].sqPts[3]);
					if( fakeMarker.isMarkerSet()){
						fakeMarker.loadARMatrix();
						
						
						ofEnableAlphaBlending();
						ofSetColor(255, 255,255);
						
						glPushMatrix();
						int pictureId = currentCards[i].pictureId;
						if (pictureId != -1){
							ofSetRectMode(OF_RECTMODE_CENTER);
							glScalef(1, -1, 1);
							ofSetColor(255, 255, 255, currentCards[i].energySmoothed*255);
							ofRect(0,0, 57, 85);
							
							glPushMatrix();
							glTranslatef(0, 0, currentCards[i].energySmoothed*30);
							glBlendFunc(GL_DST_COLOR, GL_ZERO);
							theApp->cardMan.currentlyTrackedImages[pictureId].colorImageHighRes.draw(0,-2, 60, 83);
							ofSetRectMode(OF_RECTMODE_CORNER);
							glPopMatrix();
							
						}
							glPopMatrix();
						ofDisableAlphaBlending();
						
						fakeMarker.unloadARMatrix();
					}	
					
					*/
					
					/*ofPoint pt = plane.screenToMarker(currentCards[i].center.x, currentCards[i].center.y);
					glPushMatrix();
						ofCircle(pt.x, pt.y,  10);
					glPopMatrix();
					
					glPushMatrix();
					
					ofEnableAlphaBlending();
					//ofSetColor(255,255,255,ofRandom(0,255));
					glBlendFunc(GL_SRC_ALPHA, GL_SRC_COLOR);
					
					glTranslatef(pt.x-50, pt.y+50, 0); //, <#GLfloat y#>, <#GLfloat z#>)
					glScalef(1, -1, 1); 
					ofSetColor(255,255,255);
					int pictureId = currentCards[i].pictureId;
					if (pictureId != -1){
						theApp->cardMan.currentlyTrackedImages[pictureId].colorImageHighRes.draw(0,0, 64, 80);
					}
					glPopMatrix();
					
					ofDisableAlphaBlending();
					 */
					
				}
			}
			
			// want to see all the cards! 
			if( allCards.size() > 0){
				for(int i = 0; i < allCards.size(); i++){
					
					
					glPushMatrix();
					ofEnableAlphaBlending();
					glTranslatef(0, 0, allCards[i].energySmoothed * 100);
					ofSetColor(255,0,0, allCards[i].energySmoothed * 100);
					ofBeginShape();
					for (int j = 0; j < 4; j++){
						ofPoint pt = plane.screenToMarker(allCards[i].c[j].x, allCards[i].c[j].y);
						ofVertex(pt.x, pt.y);
					}
					ofEndShape();
					ofDisableAlphaBlending();
					glPopMatrix();
					
					
					
				}
			}
		
		plane.unloadARMatrix();
	}
	
	//show the plane points we are marking out
	if(recordingPts){
		ofNoFill();
			ofSetColor(255, 0, 130);
			ofBeginShape();
				for(int i = 0; i < whichPt; i++){
					ofVertex(planePts[i].x, planePts[i].y);
				}
			ofEndShape(false);
			for(int i = 0; i < whichPt; i++){
				ofCircle(planePts[i].x, planePts[i].y, 4);
			}
		ofFill();
	}
	
	//show our debug text
	if(debug){
		drawTxtMsg("scene is: "+sceneName+" r to record points - s to save settings", 0, camH - 40);
		drawDebugTxt(0, camH - 20);	
	}

}

//---------------------------------------
void testScene::update(){
	if( pause ) return;

}

//---------------------------------------		
void testScene::keyPressed( int key ){
	if( pause ) return;

	if(key == 'r'){
		recordingPts = true;
		whichPt = 0;
		debugTxt = "set plane points - click corners clockwise from top left";		
	}

	if( key == 's'){
		xml.setValue("A:x", planePts[0].x);
		xml.setValue("A:y", planePts[0].y);
		xml.setValue("B:x", planePts[1].x);
		xml.setValue("B:y", planePts[1].y);
		xml.setValue("C:x", planePts[2].x);
		xml.setValue("C:y", planePts[2].y);	
		xml.setValue("D:x", planePts[3].x);
		xml.setValue("D:y", planePts[3].y);
		xml.saveFile("planeSettings.xml");
		debugTxt = "plane settings saved";
	}
}

//---------------------------------------
void testScene::keyReleased( int key ){
	if( pause ) return;

}
	
//---------------------------------------
void testScene::mouseMoved(int x, int y ){
	if( pause ) return;

	mx = x;
	my = y;
}

//---------------------------------------
void testScene::mouseDragged( int x, int y, int button ){

	if( recordingPts ){
		planePts[whichPt].x = x;
		planePts[whichPt].y = y;
	}
}

//---------------------------------------
void testScene::mousePressed( int x, int y, int button ){
	if( pause ) return;

	if( recordingPts ){
		planePts[whichPt].x = x;
		planePts[whichPt].y = y;
	}
}

//---------------------------------------
void testScene::mouseReleased(int x, int y, int button ){
	if( pause ) return;
	
	if( recordingPts ){
		planePts[whichPt].x = x;
		planePts[whichPt].y = y;
		whichPt++;
		if( whichPt == 4){
			recordingPts = false;
			debugTxt = "points set - hit s key to save";
			plane.setMarkerPoints(planePts[0], planePts[1], planePts[2], planePts[3]);
		}
	}

}	

